use strict;
use warnings;

use SQL::Steno;

use Test::More tests => 3;



# Fake DBI
sub test($$;$) {
    local $_ = $_[0];
    &SQL::Steno::convert;
    is $_, $_[1], $_[2] || $_[0];
}

test ':d a,b,c;t;i>5', 'select distinct a,b,c from t where i>5';

test ':q+0', 'select date_format(now()-interval mod(month(now())+11,3) month,"%Y-%m-01")';

test ';tabloid:ob2d5a9', 'select * from tabloid order by 2 desc, 5 asc, 9';

#test 'ins tabloid;a=3', 'select * from tabloid order by 2 desc, 5 asc, 9';